@@ -32,8 +32,8 @@
 
 
 
-#ifndef __BGEOT_CONVEX_HULL_H
-#define __BGEOT_CONVEX_HULL_H
+#ifndef BGEOT_CONVEX_HULL_H__
+#define BGEOT_CONVEX_HULL_H__
 
 #include <bgeot_mesh_structure.h>
 
@@ -116,7 +116,7 @@ namespace bgeot
 		= vector_from(point_list[simplex[0]], point_list[i1]);
 	      typename PT::vector_type DV
 		= vector_from(point_list[i1], point_list[i2]);
-	      base_vector d0(N), dd0(N), dV(N), ddV(N), v11(N);
+	      base_small_vector d0(N), dd0(N), dV(N), ddV(N), v11(N);
 	      for (j = 0; j < N; j++)
 	      {
 		v11[j] = 1.0;
@@ -218,4 +218,4 @@ namespace bgeot
 }  /* end of namespace bgeot.                                             */
 
 
-#endif /* __BGEOT_CONVEX_HULL_H                                           */
+#endif /* BGEOT_CONVEX_HULL_H__                                           */
