@@ -30,11 +30,12 @@
 /* *********************************************************************** */
 
 
-#ifndef __BGEOT_VECTOR_H
-#define __BGEOT_VECTOR_H
+#ifndef BGEOT_VECTOR_H__
+#define BGEOT_VECTOR_H__
 
 #include <bgeot_config.h>
-#include <gmm.h>
+#include <gmm_kernel.h>
+#include <bgeot_small_vector.h>
 
 namespace bgeot
 {
@@ -54,21 +55,21 @@ namespace bgeot
   {
     public:
 
-      typedef typename std::vector<T>::size_type size_type;
-      typedef typename std::vector<T>::iterator iterator;
-      typedef typename std::vector<T>::const_iterator const_iterator;
+    typedef typename std::vector<T>::size_type size_type;
+    typedef typename std::vector<T>::iterator iterator;
+    typedef typename std::vector<T>::const_iterator const_iterator;
 
       void out_of_range_error(void) const;
 
-      #ifdef __GETFEM_VERIFY
+      #ifdef GETFEM_VERIFY
       inline const T& operator [](size_type l) const
-      { if (l>=this->size()) out_of_range_error(); return *(this->begin()+l); }
+    { if (l>=this->size()) out_of_range_error(); return *(this->begin()+l); }
       inline T& operator [](size_type l)
-      { if (l>=this->size()) out_of_range_error(); return *(this->begin()+l); }
+    { if (l>=this->size()) out_of_range_error(); return *(this->begin()+l); }
       #endif
 
       void fill(const T &);
-      void addmul(const T &, const vsvector<T>&);
+      void addmul(const T &, const vsvector<T>&)  IS_DEPRECATED;
 
       /// Add vector w to current vector.
       vsvector<T>& operator +=(const vsvector<T>& w);
@@ -96,7 +97,7 @@ namespace bgeot
 
   template<class T> vsvector<T>::vsvector(T a0, T a1)
     : std::vector<T>(size_type(2))
-  { iterator p = this->begin(); *p++ = a0; *p++ = a1; }
+  { typename vsvector<T>::iterator p = this->begin(); *p++ = a0; *p++ = a1; }
   template<class T> vsvector<T>::vsvector(T a0, T a1, T a2)
     : std::vector<T>(size_type(3)) 
   { iterator p = this->begin(); *p++ = a0; *p++ = a1; *p++ = a2;}
@@ -108,9 +109,11 @@ namespace bgeot
   template<class T>  void vsvector<T>::out_of_range_error(void) const
   { DAL_THROW(std::out_of_range, "out of range"); }
 
-
+  /*
+    deprecated ?
+  */
   template<class T>  void vsvector<T>::addmul(const T &a, const vsvector<T> &v)
-  {                             
+  { 
     typename vsvector<T>::iterator d1 = this->begin(), e = this->end();
     const_iterator d2 = v.begin();
     if ( v.size() != this->size())
@@ -228,7 +231,9 @@ namespace bgeot
   //@}
 
   typedef vsvector<scalar_type> base_vector;
-  typedef PT<base_vector> base_node;
+  typedef small_vector<scalar_type> base_small_vector;
+  //typedef vsvector<scalar_type> base_small_vector;
+  typedef base_small_vector base_node;
 
   /*  template<class T>
     vsvector<T> operator *(const gmm::dense_matrix<T>& m, const vsvector<T>& v)
@@ -244,9 +249,20 @@ namespace bgeot
   */
   typedef gmm::dense_matrix<scalar_type> base_matrix;
 
-
+  template <class VEC_CONT> void vectors_to_base_matrix(base_matrix &G, const VEC_CONT &a) {
+    size_type P = (*(a.begin())).size(), NP = a.end() - a.begin();
+    G.resize(P, NP);
+    typename VEC_CONT::const_iterator it = a.begin(), ite = a.end();
+    base_matrix::iterator itm = G.begin();
+    for (; it != ite; ++it, itm += P)
+      std::copy((*it).begin(), (*it).end(), itm);
+  }
 }  /* end of namespace bgeot.                                           */
 
+namespace std {
+  inline void swap(bgeot::base_node& a, bgeot::base_node& b) { a.swap(b); }
+}
+
 #include <gmm_interface_bgeot.h>
 
-#endif  /* __BGEOT_VECTOR_H */
+#endif  /* BGEOT_VECTOR_H__ */
