@@ -215,8 +215,8 @@ namespace dal
 	--icard;
       }
 
-      typedef dal::reverse_iter<const_iterator> const_reverse_iterator;
-      typedef dal::reverse_iter<iterator> reverse_iterator;
+      typedef std::reverse_iterator<const_iterator> const_reverse_iterator;
+      typedef std::reverse_iterator<iterator> reverse_iterator;
       size_type size(void) const 
       { return std::max(ilast_true, ilast_false)+1;}
       
@@ -299,6 +299,7 @@ namespace dal
       while (b != e) { add(*b++); }
       return *this;
     }
+    bool contains(const dal::bit_vector &other) const;
    
   /* ********************************************************************* */
   /*									   */
@@ -308,7 +309,10 @@ namespace dal
   
     public : 
 
-      bool is_in(size_type i) const { return (*this)[i]; }
+    bool is_in(size_type i) const { 
+      if (i < ifirst_true || i > ilast_true) return false;
+      else return (((*(const bit_container*)(this))[i / WD_BIT]) & 
+		   (bit_support(1) << (i & WD_MASK))) ? true : false; }
       void add(size_type i) { (*this)[i] = true; }
     /** set the interval [i...i+nb-1] to true */
       void add(size_type i, size_type nb);
