@@ -6,6 +6,7 @@
 #include <gmm_kernel.h>
 #include <getfem_mesh_fem.h>
 #include <bgeot_sparse_tensors.h>
+#include <getfem_mat_elem_type.h>
 #include <numeric>
 #include <map>
 
@@ -64,8 +65,8 @@ namespace getfem {
      */
     virtual void update_childs_required_shape();
 
-    /* numrotation des tenseurs, telle que si i < j alors le tenseur(j) 
-       ne peut pas etre dans le sous-arbre du tenseur(i) */
+    /* numbering og tensors, such that if i < j then tensor(j)
+       cannot be in the sub-tree of tensor(i) */
     void set_number(unsigned &gcnt);
     unsigned number() const { return number_; }
   private:
@@ -81,10 +82,10 @@ namespace getfem {
     bool shape_updated_;
     tensor_ref   tr;
     tensor_shape req_shape;
-    bool frozen_; /* pour reconnaitre les resultats intermdiaires de calculs
-		     stocks dans une variable temporaire: ils ne peuvent pas tre
-		     modifis  posteriori (comme a pourrait arriver avec un 
-		     ATN_tensors_sum_scaled) */
+    bool frozen_; /* used to recognize intermediate results of
+		     computations stored in a temporary variable: they
+		     cannot be modified a posteriori (like it could
+		     happen with an ATN_tensors_sum_scaled) */
   public:
     ATN_tensor() { shape_updated_ = false; frozen_ = false; }
     bool is_shape_updated() const { return shape_updated_; }
@@ -107,8 +108,8 @@ namespace getfem {
 	child(i).merge_required_shape(req_shape);
       }
     }
-    /* ... puis reserve de la memoire si necessaire pour le stockage du tenseur 
-       dans 'reinit' (heritee ici de la class ATN)
+    /* ... then reserve some memory if necessary for tensor storage 
+       in 'reinit' (inherited here from ATN)
      */
     tensor_ref& tensor() { 
       return tr; 
@@ -174,7 +175,7 @@ namespace getfem {
       }
       mti.rewind();
       do {
-	typename VEC::iterator it = v.begin();
+	typename gmm::linalg_traits<VEC>::iterator it = gmm::vect_begin(v);
 	for (dim_type i = 0; i < mti.ndim(); ++i) it+=str[i][mti.index(i)];
 	*it += mti.p(0);
       } while (mti.qnext1());
@@ -201,9 +202,12 @@ namespace getfem {
       size_type nb_c = mf_c.nb_dof_of_element(cv);
       if (child(0).tensor().dim(0) != nb_r ||
 	  child(0).tensor().dim(1) != nb_c) {
-	ASM_THROW_TENSOR_ERROR("size mismatch for sparse matrix output: tensor dimension is " << 
-			       child(0).ranges() << ", while the elementary matrix for convex " << cv << 
-			       " should have " << nb_r << "x" << nb_c << " elements");
+	ASM_THROW_TENSOR_ERROR("size mismatch for sparse matrix output:"
+			       " tensor dimension is " << 
+			       child(0).ranges()
+			       << ", while the elementary matrix for convex "
+			       << cv << " should have " << nb_r << "x"
+			       << nb_c << " elements");
       }
       std::vector<size_type> cvdof_r(nb_r);
       std::vector<size_type> cvdof_c(nb_c);
@@ -229,18 +233,26 @@ namespace getfem {
   class base_asm_data {
   public:
     virtual size_type vect_size() const = 0;
-    virtual const scalar_type *get_data_ptr() const = 0;
+    virtual void copy_with_mti(const std::vector<tensor_strides> &, multi_tensor_iterator &) const = 0;
     virtual ~base_asm_data() {}
   };
 
   template< typename VEC > class asm_data : public base_asm_data {
-    VEC *v;
+    const VEC &v;
   public:
-    asm_data(VEC *v_) : v(v_) {}
+    asm_data(const VEC *v_) : v(*v_) {}
     size_type vect_size() const {
-      return gmm::vect_size(*v); 
+      return gmm::vect_size(v); 
+    }
+    /* used to transfert the data for the current convex to the mti of ATN_tensor_from_dofs_data */
+    void copy_with_mti(const std::vector<tensor_strides> &str, multi_tensor_iterator &mti) const {
+      typename gmm::linalg_traits<VEC>::const_iterator it;
+      do {
+        it = gmm::vect_const_begin(v);
+	for (dim_type i = 0; i < mti.ndim(); ++i) it+=str[i][mti.index(i)];
+	mti.p(0) = *it;
+      } while (mti.qnext1());
     }
-    const scalar_type *get_data_ptr() const { return &((*v)[0]); }
   };
 
   class base_asm_vec {
@@ -412,7 +424,7 @@ namespace getfem {
   /* main class for generic assembly */
   class generic_assembly : public asm_tokenizer {
     std::deque<const mesh_fem *> mftab;/* list of the mesh_fem used in the computation */
-  
+    std::deque<pnonlinear_elem_term> innonlin;  /* alternatives to base, grad, hess in comp() for non-linear computations) */
     std::deque<base_asm_data*> indata;              /* data sources */
     std::deque<base_asm_vec*> outvec;               /* vectors in which is done the assembly */
     std::deque<base_asm_mat*> outmat;               /* matrices in which is done the assembly */
@@ -458,19 +470,30 @@ namespace getfem {
 
     void set(const std::string& s_) { set_str(s_); }
     const std::deque<const mesh_fem*>& mf() const { return mftab; }
+    const std::deque<pnonlinear_elem_term> nonlin() const { return innonlin; }
     const std::deque<base_asm_data*>& data() const { return indata; }
     const std::deque<base_asm_vec*>& vec() const { return outvec; }
     const std::deque<base_asm_mat*>& mat() const { return outmat; }
     void push_mf(const mesh_fem& mf_) { mftab.push_back(&mf_); }
-    template< typename VEC > void push_data(VEC& d) { 
+    void push_nonlinear_term(pnonlinear_elem_term net) {
+      innonlin.push_back(net);
+    }
+    template< typename VEC > void push_data(const VEC& d) { 
       indata.push_back(new asm_data<VEC>(&d)); 
     }
     template< typename VEC > void push_vec(VEC& v) { 
-      asm_vec<VEC> *pv = new asm_vec<VEC>(&v);
+      asm_vec<VEC> *pv = new asm_vec<VEC>(&(gmm::linalg_cast(v)));
       outvec.push_back(pv);
     }
+    template< typename VEC > void push_vec(const VEC& v) { 
+      asm_vec<VEC> *pv = new asm_vec<VEC>(&(gmm::linalg_cast(v)));
+      outvec.push_back(pv);
+    }
+    template< typename MAT > void push_mat(const MAT& m) { 
+      outmat.push_back(new asm_mat<MAT>(&(gmm::linalg_cast(m)))); 
+    }
     template< typename MAT > void push_mat(MAT& m) { 
-      outmat.push_back(new asm_mat<MAT>(&m)); 
+      outmat.push_back(new asm_mat<MAT>(&(gmm::linalg_cast(m)))); 
     }
 
     void set_vec_factory(base_vec_factory *fact) { vec_fact = fact; }
@@ -485,8 +508,9 @@ namespace getfem {
       t->set_name(mark_txt());
       outvars.push_back(t); return t;
     }
-    const mesh_fem& do_mf_arg();
+    const mesh_fem& do_mf_arg(std::vector<const mesh_fem*> *multimf = 0);
     void do_dim_spec(vdim_specif_list& lst);
+    std::string do_comp_red_ops();
     ATN_tensor* do_comp();
     ATN_tensor* do_data();
     std::pair<ATN_tensor*, std::string> do_red_ops(ATN_tensor* t);
