@@ -34,13 +34,13 @@
 #define GETFEM_FEM_H__
 
 #include <bgeot_geometric_trans.h>
+#include <bgeot_precomp.h>
 #include <getfem_integration.h>
 #include <getfem_poly_composite.h>
 #include <getfem_precomp.h>
 #include <deque>
 
-namespace getfem
-{
+namespace getfem {
   /************************************************************************/
   /*	Class for description of an interpolation dof.                    */
   /************************************************************************/
@@ -62,7 +62,7 @@ namespace getfem
   /// Description of a unique dof of mean value type.
   pdof_description mean_value_dof(dim_type);
   
-  pdof_description already_numerate_dof(dim_type);
+  pdof_description global_dof(dim_type);
   /// Product description of the descriptions *pnd1 and *pnd2.
   pdof_description product_dof(pdof_description, pdof_description);
 
@@ -87,60 +87,65 @@ namespace getfem
   
   class virtual_fem;
   typedef const virtual_fem * pfem;
+
+  class fem_interpolation_context;
   
-  class virtual_fem
-  {
+  class virtual_fem {
+
   protected :
 
-    std::vector<pdof_description> dof_types_;
+    mutable std::vector<pdof_description> dof_types_;
     bgeot::convex_structure cvs_node;
     bgeot::convex<base_node> cv_node;
     mutable bgeot::pstored_point_tab pspt;
     mutable bool pspt_valid;
     bgeot::pconvex_ref cvr; // reference element.
-    dim_type ntarget_dim;
+    mutable dim_type ntarget_dim, dim_;
     bool is_equiv, is_lag, is_pol, is_polycomp, real_element_defined;
     short_type es_degree, hier_raff;
     
   public :
     /// Number of degrees of freedom.
-    virtual size_type nb_dof(void) const { return dof_types_.size(); }
-    virtual size_type nb_base(void) const { return nb_dof(); }
+    virtual size_type nb_dof(size_type) const
+    { return dof_types_.size(); }
+    virtual size_type nb_base(size_type cv) const
+    { return nb_dof(cv); }
     /// Number of components (nb_dof() * dimension of the target space).
-    size_type nb_base_components(void) const
-      { return nb_base() * ntarget_dim; }
-    size_type nb_components(void) const
-      { return nb_dof() * ntarget_dim; }
+    size_type nb_base_components(size_type c) const
+      { return nb_base(c) * ntarget_dim; }
+    size_type nb_components(size_type c) const
+      { return nb_dof(c) * ntarget_dim; }
     /// Gives the array of pointer on dof description.
     const std::vector<pdof_description> &dof_types(void) const 
       { return dof_types_; }
     short_type hierarchical_raff(void) const { return hier_raff; }
     /// dimension of the reference element.
-    dim_type dim(void) const { return cvr->structure()->dim(); }
+    dim_type dim(void) const { return dim_; }
+    dim_type &dim(void) { return dim_; }
     /// dimension of the target space.
     dim_type target_dim(void) const { return ntarget_dim; }
-    /// Gives the convex structure of the reference element nodes.
-    bgeot::pconvex_structure structure(void) const
-      { return cv_node.structure(); }
     /// Gives the convex of the reference element.
-    bgeot::pconvex_structure basic_structure(void) const
-      { return cvr->structure(); }
+    virtual bgeot::pconvex_ref ref_convex(size_type) const { return cvr; }
+    bgeot::pconvex_ref &mref_convex() { return cvr; }
     /// Gives the convex of the reference element.
-    bgeot::pconvex_ref ref_convex(void) const { return cvr; }
-    bgeot::pconvex_ref &ref_convex(void) { return cvr; }
+    bgeot::pconvex_structure basic_structure(size_type cv) const
+    { return ref_convex(cv)->structure(); }
     /// Gives the convex representing the nodes on the reference element.
-    const bgeot::convex<base_node> &node_convex(void) const
+    virtual const bgeot::convex<base_node> &node_convex(size_type) const
       { return cv_node; }
-    /// Gives the node corresponding to the dof i.
-    const base_node &node_of_dof(size_type i) const
-      { return cv_node.points()[i];}
-    bgeot::pstored_point_tab node_tab(void) const { 
+    /// Gives the convex structure of the reference element nodes.
+    bgeot::pconvex_structure structure(size_type cv) const
+    { return node_convex(cv).structure(); }
+    virtual bgeot::pstored_point_tab node_tab(size_type) const { 
       if (!pspt_valid) {
 	pspt = bgeot::store_point_tab(cv_node.points());
 	pspt_valid = true;
       }
       return pspt;
     }
+    /// Gives the node corresponding to the dof i.
+    const base_node &node_of_dof(size_type cv, size_type i) const
+      { return (*(node_tab(cv)))[i];}
     bool is_on_real_element(void) const { return real_element_defined; }
     bool is_equivalent(void) const { return is_equiv; }
     bool need_G(void) const
@@ -164,21 +169,26 @@ namespace getfem
      *  for non-equivalent elements.
      *  This method take all cases into account (non tau-equivalent element)
      */
-    virtual void interpolation(const base_node &x, const base_matrix &G,
-			       bgeot::pgeometric_trans pgt,
-			       const base_vector &coeff, 
-			       base_vector &val) const = 0;
+    template<typename CVEC, typename VVEC> 
+    void interpolation(const fem_interpolation_context& c, 
+		       const CVEC& coeff, VVEC &val, dim_type Qdim) const;
+
+    /** Function which interpolates in a arbitrary point x given on the
+     *  reference element.
+     *  This method take all cases into account (non tau-equivalent element)
+     *  the result is the matrix M such that M * coeff gives the value of
+     *  the interpolation (i.e. the result of the latter function).
+     */
+    template <typename MAT>
+    void interpolation(const fem_interpolation_context& c, 
+		       MAT &M, dim_type Qdim) const;
+
     /** Function which interpolates in the ii th point of pfp. coeff is the
      *  vector of coefficient relatively to the shape functions. G and pgt
      *  represent the geometric transformation for non-equivalent elements.
-     *  Qdim take into account a vectorisation of the element.
+     *  Qdim takes into account a vectorisation of the element.
      *  This method take all cases into account (non tau-equivalent element)
      */
-    virtual void interpolation(pfem_precomp pfp, size_type ii,
-			       const base_matrix &G,
-			       bgeot::pgeometric_trans pgt, 
-			       const base_vector &coeff, 
-			       base_vector &val, dim_type Qdim=1) const;
     /** Function which interpolates the gradient in a arbitrary point x
      *  given on the reference element. coeff is the vector of coefficient
      *  relatively to the shape functions. G and pgt
@@ -188,18 +198,14 @@ namespace getfem
      *  and correction of the gradient via the gradient of the geometric
      *  transformation).
      *  This function is essentially used by virtual_link_fem.
+     *  The gradient is stored in the rows of val, which should be
+     *  a 'Qdim' x 'N' matrix.
      */
-    virtual void interpolation_grad(const base_node &x,
-				    const base_matrix &G,
-				    bgeot::pgeometric_trans pgt,
-				    const base_vector &coeff,
-				    base_matrix &val) const;
-    //     virtual void interpolation_grad(pfem_precomp pfp, size_type ii,
-    // 				    const base_matrix &G,
-    // 				    bgeot::pgeometric_trans pgt, 
-    // 				    const base_vector &coeff,
-    // 				    base_matrix &val) const;
-
+    template<typename CVEC, typename VMAT> 
+    void interpolation_grad(const fem_interpolation_context& c, 
+			    const CVEC& coeff, VMAT &val,
+			    dim_type Qdim=1) const;
+      
     /** Gives the value of all components of the base functions at the
      *  point x of the reference element. Basic function used essentially
      *  by fem_precomp.
@@ -220,9 +226,9 @@ namespace getfem
      *  Used by elementary computations.
      *  Matrix M for non tau-equivalent elements not taken into account.
      */
-    virtual void real_base_value(pgeotrans_precomp pgp, pfem_precomp pfp,
-				 size_type ii, const base_matrix &G,
-				 base_tensor &t, size_type elt) const;
+    virtual void real_base_value(const fem_interpolation_context &c, 
+				 base_tensor &t) const;
+
     /** Gives the value of all gradients on the real element of the components
      *  of the base functions at the point ii of the pgp possibly using
      *  information on pfp, G and B. B is the matrix wich transforms the
@@ -230,34 +236,31 @@ namespace getfem
      *  Used by elementary computations.
      *  Matrix M for non tau-equivalent elements not taken into account.
      */
-    virtual void real_grad_base_value(pgeotrans_precomp pgp, pfem_precomp pfp,
-				      size_type ii, const base_matrix &G,
-				      const base_matrix &B,
-				      base_tensor &t, size_type elt) const;
-    /** Gives the value of all hessians on the real element of the components
+    virtual void real_grad_base_value(const fem_interpolation_context &c, 
+				      base_tensor &t) const;
+/** Gives the value of all hessians on the real element of the components
      *  of the base functions at the point ii of the pgp possibly using
      *  information on pfp, G, B2 and B32. B2 and B32 are the matrices used
      *  to transform a Hessian on reference element to a Hessian on real
      *  element. Used by elementary computations.
      *  Matrix M for non tau-equivalent elements not taken into account.
      */
-    virtual void real_hess_base_value(pgeotrans_precomp pgp, pfem_precomp pfp,
-				      size_type ii, const base_matrix &G,
-				      const base_matrix &B3,
-				      const base_matrix &B32,
-				      base_tensor &t, size_type elt) const;
 
+    virtual void real_hess_base_value(const fem_interpolation_context &c, 
+				      base_tensor &t) const;
     
-    virtual size_type index_of_already_numerate_dof(size_type, size_type) const
+    virtual size_type index_of_global_dof(size_type, size_type) const
       { DAL_THROW(internal_error, "internal error."); }
 
     virtual_fem(void) { 
-      ntarget_dim = 1; is_equiv = is_pol = is_polycomp = is_lag = false;
+      ntarget_dim = 1; dim_ = 1; 
+      is_equiv = is_pol = is_polycomp = is_lag = false;
       pspt_valid = false; hier_raff = 0; real_element_defined = false;
       es_degree = 5;
     }
 
-    void add_node(const pdof_description &d, const base_node &pt) ;
+    void add_node(const pdof_description &d, const base_node &pt,
+		  bool all_faces = false) ;
     void init_cvs_node(void);
     void unfreeze_cvs_node(void);
 
@@ -269,6 +272,7 @@ namespace getfem
       pspt = 0;
       pspt_valid = false;
       cvr = f.cvr;
+      dim_ = f.dim_;
       ntarget_dim = f.ntarget_dim;
       is_equiv = f.is_equiv;
       is_lag = f.is_lag;
@@ -282,10 +286,160 @@ namespace getfem
     virtual_fem(const virtual_fem &f) { *this = f; }
 
     virtual ~virtual_fem() {}
-
   };
   
-  
+  /* the fem_interpolation_context structure is passed as the argument
+     of fem interpolation functions. This structure can be partially
+     filled (for example the xreal will be computed if needed as long
+     as pgp+ii is known)
+  */
+  class fem_interpolation_context :
+    public bgeot::geotrans_interpolation_context {
+
+    mutable base_matrix M_;
+    pfem pf_;
+    pfem_precomp pfp_;
+    size_type convex_num_;
+  public:
+    bool have_pfp() const { return pfp_ != 0; }
+    bool have_pf() const { return pf_ != 0; }
+    const base_matrix& M() const;
+    void base_value(base_tensor& t) const;
+    void grad_base_value(base_tensor& t) const;
+    void hess_base_value(base_tensor& t) const;
+    const pfem pf() const { return pf_; }
+    size_type convex_num() const;
+    pfem_precomp pfp() const { return pfp_; }
+    void set_pfp(pfem_precomp newpfp);
+    void set_pf(pfem newpf);
+    fem_interpolation_context();
+    fem_interpolation_context(bgeot::pgeotrans_precomp pgp__, 
+			      pfem_precomp pfp__, size_type ii__, 
+			      const base_matrix& G__, 
+			      size_type convex_num__);
+    fem_interpolation_context(bgeot::pgeometric_trans pgt__, 
+			      pfem_precomp pfp__, size_type ii__, 
+			      const base_matrix& G__, 
+			      size_type convex_num__);
+    fem_interpolation_context(bgeot::pgeometric_trans pgt__,
+			      pfem pf__,
+			      const base_node& xref__,
+			      const base_matrix& G__,
+			      size_type convex_num__);
+  };
+
+  template <typename CVEC, typename VVEC>
+  void virtual_fem::interpolation(const fem_interpolation_context& c, 
+				  const CVEC& coeff, VVEC &val,
+				  dim_type Qdim) const {
+    size_type Qmult = size_type(Qdim) / target_dim();
+    if (gmm::vect_size(val) != Qdim)
+      DAL_THROW(dimension_error, "dimensions mismatch");
+    size_type R = nb_dof(c.convex_num()), RR = nb_base(c.convex_num());
+    
+    gmm::clear(val);
+    base_tensor Z; real_base_value(c,Z);
+    for (size_type j = 0; j < RR; ++j) {
+      for (size_type q = 0; q < Qmult; ++q) {
+	typename gmm::linalg_traits<CVEC>::value_type co = 0.0;
+	if (is_equivalent())
+	  co = coeff[j*Qmult+q];
+	else
+	  for (size_type i = 0; i < R; ++i)
+	    co += coeff[i*Qmult+q] * c.M()(i, j);	  
+	for (size_type r = 0; r < target_dim(); ++r)
+	  val[r + q*target_dim()] += co * Z[j + r*R];
+      } 
+    }
+  }
+
+  template <typename MAT>
+  void virtual_fem::interpolation(const fem_interpolation_context& c, 
+				  MAT &M, dim_type Qdim) const {
+    size_type Qmult = size_type(Qdim) / target_dim();
+    size_type R = nb_dof(c.convex_num()), RR = nb_base(c.convex_num());
+    if (gmm::mat_nrows(M) != Qdim || gmm::mat_ncols(M) != R*Qmult)
+      DAL_THROW(dimension_error, "dimensions mismatch");
+    
+    gmm::clear(M);
+    base_tensor Z; real_base_value(c,Z);
+    for (size_type j = 0; j < RR; ++j) {
+      for (size_type q = 0; q < Qmult; ++q) {
+	for (size_type r = 0; r < target_dim(); ++r)
+	  if (is_equivalent())
+	    M(r+q*target_dim(), j*Qmult+q) = Z[j + r*R];
+	  else
+	    for (size_type i = 0; i < R; ++i)
+	      M(r+q*target_dim(), i*Qmult+q) = c.M()(i, j) * Z[j + r*R];
+      } 
+    }
+  }
+
+
+  template<typename CVEC, typename VMAT> 
+  void virtual_fem::interpolation_grad(const fem_interpolation_context& c, 
+			  const CVEC& coeff, VMAT &val, dim_type Qdim) const {
+    typedef typename gmm::linalg_traits<CVEC>::value_type T;
+    size_type Qmult = size_type(Qdim) / target_dim();
+    dim_type N = c.N();
+    if (gmm::mat_ncols(val) != N || gmm::mat_nrows(val) != Qdim)
+      DAL_THROW(dimension_error, "dimensions mismatch");
+    
+    dim_type P = dim();
+    base_tensor t;
+    size_type R = nb_dof(c.convex_num()), RR = nb_base(c.convex_num());      
+    
+    gmm::clear(val);
+    if (!is_on_real_element()) { // optimized case
+      if (!c.have_pfp()) {
+	grad_base_value(c.xref(), t);
+      } else {
+	t = c.pfp()->grad(c.ii());
+      }
+      gmm::dense_matrix<T> val2(P, Qdim);
+      gmm::clear(val2);
+      for (size_type q = 0; q < Qmult; ++q) {
+	base_tensor::iterator it = t.begin();	
+	for (size_type k = 0; k < P; ++k)
+	  for (size_type r = 0; r < target_dim(); ++r)
+	    for (size_type j = 0; j < RR; ++j, ++it) {
+	      T co = 0.0;
+	      if (is_equivalent())
+		co = coeff[j*Qmult+q];
+	      else
+		for (size_type i = 0; i < R; ++i)
+		  co += coeff[i*Qmult+q] * c.M()(i, j);	      
+	      val2(k, r + q*target_dim()) += co * (*it);
+	    }
+      }
+      gmm::mult(c.B(), val2, gmm::transposed(val));
+      // replaced by the loop because gmm does not support product of a complex<> matrix with a real matrix
+      /*for (size_type i=0; i < Qdim; ++i)
+	for (size_type j=0; j < N; ++j) {
+	  T s = 0;
+	  for (size_type k=0; k < P; ++k) s += c.B()(j,k)*val2(k,i);
+	  val(i,j) = s;
+	}
+      */
+    } else {
+      real_grad_base_value(c, t);
+      for (size_type q = 0; q < Qmult; ++q) {
+	base_tensor::iterator it = t.begin();
+	for (size_type k = 0; k < N; ++k)
+	  for (size_type r = 0; r < target_dim(); ++r)
+	    for (size_type j = 0; j < RR; ++j, ++it) {
+	      T co = 0.0;
+	      if (is_equivalent())
+		co = coeff[j*Qmult+q];
+	      else
+		for (size_type i = 0; i < R; ++i)
+		  co += coeff[i*Qmult+q] * c.M()(i, j);	      
+	      val(r + q*target_dim(), k) += co * (*it);
+	    }
+      }
+    }
+  }
+
   
   template <class FUNC> class fem : public virtual_fem {
   protected :
@@ -296,24 +450,11 @@ namespace getfem
     /// Gives the array of basic functions (components).
     const std::vector<FUNC> &base(void) const { return base_; }
     std::vector<FUNC> &base(void) { return base_; }
-    
-    /* just to please for HP aCC and SGI CC */
-    virtual void interpolation(pfem_precomp pfp, size_type ii,
-			       const base_matrix &G,
-			       bgeot::pgeometric_trans pgt, 
-			       const base_vector &coeff, 
-			       base_vector &val, dim_type Qdim=1) const { 
-      virtual_fem::interpolation(pfp,ii,G,pgt,coeff,val,Qdim); 
-    }
-    void interpolation(const base_node &x, const base_matrix &G, 
-		       bgeot::pgeometric_trans pgt,
-		       const base_vector &coeff, base_vector &val) const;
-
     void base_value(const base_node &x, base_tensor &t) const {
       bgeot::multi_index mi(2);
-      mi[1] = target_dim(); mi[0] = nb_base();
+      mi[1] = target_dim(); mi[0] = nb_base(0);
       t.adjust_sizes(mi);
-      size_type R = nb_base_components();
+      size_type R = nb_base_components(0);
       base_tensor::iterator it = t.begin();
       for (size_type  i = 0; i < R; ++i, ++it)
 	*it = base_[i].eval(x.begin());
@@ -321,9 +462,9 @@ namespace getfem
     void grad_base_value(const base_node &x, base_tensor &t) const {
       bgeot::multi_index mi(3);
       dim_type n = dim();
-      mi[2] = n; mi[1] = target_dim(); mi[0] = nb_base();
+      mi[2] = n; mi[1] = target_dim(); mi[0] = nb_base(0);
       t.adjust_sizes(mi);
-      size_type R = nb_base_components();
+      size_type R = nb_base_components(0);
       base_tensor::iterator it = t.begin();
       for (dim_type j = 0; j < n; ++j)
 	for (size_type i = 0; i < R; ++i, ++it)
@@ -332,9 +473,9 @@ namespace getfem
     void hess_base_value(const base_node &x, base_tensor &t) const {
       bgeot::multi_index mi(4);
       dim_type n = dim();
-      mi[3] = n; mi[2] = n; mi[1] = target_dim(); mi[0] = nb_base();
+      mi[3] = n; mi[2] = n; mi[1] = target_dim(); mi[0] = nb_base(0);
       t.adjust_sizes(mi);
-      size_type R = nb_base_components();
+      size_type R = nb_base_components(0);
       base_tensor::iterator it = t.begin();
       for (dim_type k = 0; k < n; ++k)
 	for (dim_type j = 0; j < n; ++j)
@@ -351,38 +492,15 @@ namespace getfem
   typedef const fem<base_poly> * ppolyfem;
   typedef const fem<polynomial_composite> * ppolycompfem;
   
-  template <class FUNC>
-  void fem<FUNC>::interpolation(const base_node &x, const base_matrix &G,
-				bgeot::pgeometric_trans pgt, 
-				const base_vector &coeff,
-				base_vector &val) const { 
-    // optimisable.   verifier et faire le vectoriel
-    base_matrix M;
-    if (val.size() != target_dim())
-      DAL_THROW(dimension_error, "dimensions mismatch");
-    
-    size_type R = nb_dof(), RR = nb_base();
-    
-    if (!is_equivalent()) { M.resize(RR, R); mat_trans(M, G, pgt); }
-    
-    val.fill(0.0);
-    for (size_type j = 0; j < RR; ++j) {
-      scalar_type co = 0.0;
-      if (is_equivalent())
-	co = coeff[j];
-      else
-	for (size_type i = 0; i < R; ++i)
-	  co += coeff[i] * M(i, j);
-      
-      for (size_type r = 0; r < target_dim(); ++r)
-	val[r] += co * double(base()[j + r*R].eval(x.begin()));
-    } 
-  }
-  
   /** Gives a pointer on the structures describing the more classical fem
    *  of degree k on a geometric convex cvs (coming from the geometric trans).
    */
   pfem classical_fem(bgeot::pgeometric_trans pg, short_type k);
+  /** Gives a pointer on the structures describing the more classical
+   *  discontinuous fem of degree k on a geometric convex cvs (coming 
+   *  from the geometric trans).
+   */
+  pfem classical_discontinuous_fem(bgeot::pgeometric_trans pg, short_type k);
 
   pfem fem_descriptor(std::string name);
   /*  List of elements :
@@ -427,13 +545,6 @@ namespace getfem
   pfem PK_prism_fem(size_type n, short_type k);
 
   std::string name_of_fem(pfem p);
-
-  
-  class mesh_fem;
-  pfem virtual_link_fem(mesh_fem &mf1, mesh_fem &mf2,
-			pintegration_method pim);
-  pfem virtual_link_fem_with_gradient(mesh_fem &mf1, mesh_fem &mf2,
-				      pintegration_method pim);
   
 }  /* end of namespace getfem.                                            */
 
