@@ -1,5 +1,5 @@
 $er = 0;
-open F, "geo_trans 2>&1 |" or die;
+open F, "./geo_trans 2>&1 |" or die;
 while (<F>) {
   # print $_;
   if ($_ =~ /error has been detected/)
@@ -9,6 +9,5 @@ while (<F>) {
     print $_, <F>;
   }
 }
+close(F); if ($?) { exit(1); }
 if ($er == 1) { exit(1); }
-`geo_trans`;
-if ($?) { exit(1); }
